var Observable = require("FuseJS/Observable")

var rowItemsToAddOnScroll = 1
var maxNrofrowItemsRendered = 36
var data

var listStart
var listEnd

exports.listHeaderData = Observable()
exports.listData = Observable()
exports.listRowSpacing = Observable(1)
exports.headerItemHeight = Observable(36)
exports.rowItemHeight = Observable(60)
exports.currentrowItemLabel = Observable()

var today

function generateRowItem (startAt) {
    var cursor = startAt
               
    return {
        type: "row",
        items: {
            0: startAt,
            1: startAt + 1,
            2: startAt + 2,
            3: startAt + 3,
            4: startAt + 4,
            5: startAt + 5,
            6: startAt + 6
        },
        height: exports.rowItemHeight.value
    }
}

function getHeaderItem (startAt, height, prevHeader) {
    if (prevHeader) {
        prevHeader.endShift.value = height - exports.headerItemHeight.value - exports.listRowSpacing.value
        prevHeader.shiftLength.value = prevHeader.endShift.value - prevHeader.startShift.value
    }

    if (startAt) {
        
        return {
            type: 'header',
            label: 'Header ' + startAt,
            startShift: Observable(height),
            endShift: Observable(),
            shiftLength: Observable(),
            height: exports.headerItemHeight.value
        }
    }
}

function _createItems (fromVal, toVal, latestHeader, currentHeight) {
    outp = []
    var cursor = fromVal
    while (cursor <= toVal) {
        if (cursor % (7 * 5) === 0) {
            var headerItem = getHeaderItem(cursor, currentHeight, latestHeader)
            // We only want to mutate last one changed, the rest are new 
            outp.push(headerItem)
            latestHeader = headerItem
            currentHeight += headerItem.height + exports.listRowSpacing.value
        }
        var rowItem = generateRowItem(cursor)
        outp.push(rowItem)
        currentHeight += rowItem.height + exports.listRowSpacing.value
        cursor += 7
    }
    return {
        cursor: cursor,
        currentHeight: currentHeight,
        latestHeader: latestHeader,
        items: outp 
    }
}

function refresh(renderFromVal, renderToVal) {
    // Always add complete headerItems, so choosing first day in headerItem
    var fromVal = renderFromVal
    // Choose last day of headerItem 
    var toVal = renderToVal

    // Remove leading items if scrolling down
    if (exports.listData.length > 0) {
        var itemsToRemove = 0
        var cursor = listStart
        while (cursor < fromVal) {
            var tmp = exports.listData.getAt(itemsToRemove) 
            if (tmp.type === 'header') {
                itemsToRemove += 2
            } else {
                itemsToRemove += 1
            }
            cursor += 7
        }
        if (itemsToRemove > 0) exports.listData.removeRange(0, itemsToRemove)
    }
    var t_remove = Date.now()
    // Add leading items if scrolling up or list is empty
    var currentIndex = 0
    var currentHeight = 0
    if (listStart) {
        // Since we render including toVal we need to go back a rowItem if we use listStart
        var rendertoVal = listStart - 7
    } else {
        var rendertoVal = toVal
    }

    // Only add leading items if fromVal < rendertoVal 
    if (fromVal < renderToVal) {
        var result = _createItems(fromVal, rendertoVal, undefined, currentHeight)
        var currentHeight = result.currentHeight
        var latestHeader = result.latestHeader
        var cursor = result.cursor
        if (result.items.length > 0) {
            exports.listData.insertAll(0, result.items)
            currentIndex = result.items.length
        }
        var t_addLeading = Date.now()
    }

    // Update startShift, endShift and shiftLength of existing items
    while (listEnd && cursor < listEnd && cursor < toVal) {
        var tmp = exports.listData.getAt(currentIndex)
        if (tmp && tmp.type === 'header') {
            tmp.startShift.value = currentHeight
            getHeaderItem(cursor, currentHeight, latestHeader)
            latestHeader = tmp
            currentHeight += tmp.height + exports.listRowSpacing.value
            currentIndex++
        } else {
            currentHeight += tmp.height + exports.listRowSpacing.value
            currentIndex++
            cursor += 7
        }
    }

    if (cursor < listEnd && ((currentIndex + 1) < exports.listData.length)) {
        // if toVal < listEndDate remove rest of items 
        exports.listData.removeRange(currentIndex + 1, exports.listData.length - (currentIndex + 1))
    } else if (cursor < toVal) {
        // Otherwise add trailing items if scrolling down
        var appendResult = _createItems(cursor, toVal, latestHeader, currentHeight)
        var currentHeight = appendResult.currentHeight
        var latestHeader = appendResult.latestHeader
        var cursor = appendResult.cursor
        if (appendResult.items.length > 0) {
            exports.listData.addAll(appendResult.items)
            currentIndex = exports.listData.length
        }
    }

    // Fix scrolling of last item, apparently it is needed
    getHeaderItem(undefined, currentHeight, latestHeader)
    listStart = fromVal
    listEnd = cursor
}

var isFirstRender = true
exports.gotoCurrentrowItem = function () {
    if (isFirstRender) view.seekTo(0, (6 - 1) * 61 + 31)
    console.log(">>> Called gotoCurrentrowItem")
    isFirstRender = false
}

exports.gotoCurrentrowItemSmoothly = function () {
    console.log(">>> Called gotoCurrentrowItemSmoothly")
    _renderCurrentrowItem()
}

var haltAdding = false

exports.doLoadAfter = function () {
    if (haltAdding) return

    var toVal = listEnd + (7 * rowItemsToAddOnScroll) 
    var fromVal = toVal + (-7 * maxNrofrowItemsRendered)
    console.log("vvv Load after: " + fromVal + "-" + toVal)

    refresh(fromVal, toVal)
    atEnd.check()
}

exports.doLoadBefore = function () {
    if (haltAdding) return

    var fromVal = listStart + (-7 * rowItemsToAddOnScroll)
    var toVal = fromVal + (7 * maxNrofrowItemsRendered)
    console.log("^^^ Load before: " + fromVal + "-" + toVal)

    refresh(fromVal, toVal)
    atStart.check()
}

module.exports.doToggleAdding = function () {
    haltAdding = !haltAdding
    if (haltAdding) {
        console.log("## STOP adding rows! ##")
    } else {
        console.log("## START adding rows! ##")
    }
}

console.log("*********************")
console.log("** Setup StartPage **")
console.log("*********************")

function _renderCurrentrowItem (firstLoad) {
    var fromVal = 7 * 1000 + (-7 * (firstLoad ? 6 : rowItemsToAddOnScroll))
    var toVal = fromVal + (7 * maxNrofrowItemsRendered)
    refresh(fromVal, toVal)
}
_renderCurrentrowItem(true)

module.dispose = function () {
    console.log("### Dispose of StartPage")
}
